///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                        Screen mode change routines                        //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Screen_Utils;

interface

uses
	Windows, Dialogs, Controls, Zone_settings;

function  set_256_graphics_mode(Options:Zone_Options; disable_change:boolean) : boolean;
procedure restore_original_screen_mode;
procedure ScreenCenterWindow(Wnd:integer);

{$IFDEF SCREENSAVER }
	// to avoid nasty screen mode changes all the time during config & preview
	// we'll use a special flag....(only in screensaver)
var
	special_config_mode_flag : boolean;
{$ENDIF }

implementation

var
	display_changed : boolean;
	forced          : boolean;

function set_256_graphics_mode(Options:Zone_Options; disable_change:boolean) : boolean;
var bits_pixel,X,Y : integer;
		screenDC       : hDC;
		lp1,dlgres     : integer;
		dm,new_display : TDeviceMode;
		found          : boolean;
		s              : string;
begin
  result          := false;
  display_changed := false;
  forced          := false;
  // get current screen details
  screenDC   := GetDC(GetDesktopWindow);
  bits_pixel := GetDeviceCaps(screenDC,BITSPIXEL);
  X          := GetDeviceCaps(screenDC,HORZRES);
  Y          := GetDeviceCaps(screenDC,VERTRES);
  ReleaseDC(GetDesktopWindow,screenDC);
  if bits_pixel=8 then begin
	result := true;
	exit;
  end;

	dm.dmSize := SizeOf(TDeviceMode);
	// look for a compatible 8 bit mode
	lp1 := 0; found := false;
	while EnumDisplaySettings(nil,lp1,dm) do begin
		if (dm.dmBitsPerPel=8) and (X=dm.dmPelsWidth) and (Y=dm.dmPelsHeight) then begin
			found       := true;
			new_display := dm;
		end;
		inc(lp1);
	end;
	if not found then begin
		Options.error_flag   := Options.error_flag + [mode_not_found];
		Options.error_string := Options.error_string +
			'Zonerings couldn''t find a compatible 256 colour display mode' + eol;
		exit;
	end;

	// Let's try and change screen mode (NB screensaver module won't show message
	// boxes to prompt before changing mode)
	{$IFNDEF SCREENSAVER }
		if Options.Prompt_change then begin
			s := '  Zonerings32 uses palette animation to achieve colour cycling effects'#13+
					 '                     you need to use a 256 colour display mode.       '#13+
					 '     Do you wish to temporarily switch to a 256 colour display mode ? ';
			dlgres := MessageDlg(s, mtConfirmation, [mbYes,mbNo], 0);
		end else dlgres := mrYes;
	{$ELSE }
		if Options.Prompt_change then dlgres := mrNo
		else dlgres := mrYes;
		if disable_change then begin
			special_config_mode_flag := true;
			result := true;
			exit;
		end;
	{$ENDIF }
	case dlgres of
		mrYes : begin
			new_display.dmFields := DM_BITSPERPEL + DM_PELSWIDTH + DM_PELSHEIGHT;
			case ChangeDisplaySettings(new_display,0) of
				DISP_CHANGE_SUCCESSFUL	: begin
					display_changed := true;
					result          := true;
				end;
				DISP_CHANGE_RESTART : begin
					{$IFNDEF SCREENSAVER }
						if Options.Prompt_force then begin
							s := '  Zonerings tried to change screen mode, but Windows requested a restart.'#13+
									 '               Do you wish to try and force a colour change ? '#13+
									 '(This may cause your windows colours to be badly affected until your next reboot)';
							dlgres := MessageDlg(s, mtConfirmation, [mbYes,mbNo], 0);
						end else dlgres := mrYes;
					{$ELSE }
						if Options.Prompt_force then dlgres := mrNo
						else dlgres := mrYes;
					{$ENDIF }
					case dlgres of
						mrYes : begin
							case ChangeDisplaySettings(new_display,$80000000 { undocumented CDS_FORCE }) of
								DISP_CHANGE_SUCCESSFUL : begin
									display_changed := true;
									forced          := true;
									result          := true;
								end
								else begin
									Options.error_flag   := Options.error_flag + [mode_change_restart];
									Options.error_string := Options.error_string +
										'Zonerings couldn''t change into 256 colour mode without a reboot' + eol;
								end;
							end;
						end;
						mrNo : begin
							Options.error_flag   := Options.error_flag + [mode_change_restart];
							Options.error_string := Options.error_string +
								'Zonerings will not run unless you enable automatic (force) screen mode change' + eol;
						end;
					end;
				end;
				else begin
					Options.error_flag   := Options.error_flag + [mode_change_failed];
					Options.error_string := Options.error_string +
						'Zonerings tried to change screen mode but failed :(' + eol;
				end;
			end;
		end;
		mrNo : begin
			Options.error_flag   := Options.error_flag + [mode_change_not_allowed];
			Options.error_string := Options.error_string +
				'Zonerings will not run unless you enable automatic screen mode change' + eol;
		end;
	end;
end;

procedure restore_original_screen_mode;
var dummy : ^TDeviceMode;
begin
  dummy := nil;
	if display_changed then begin
		if forced then ChangeDisplaySettings(dummy^,$80000000)
		else ChangeDisplaySettings(dummy^,0);
	end;
end;

procedure ScreenCenterWindow(Wnd:integer);
var Rect: TRect;
begin
	GetWindowRect(Wnd, Rect);
	SetWindowPos(Wnd, 0,
		(GetSystemMetrics(SM_CXSCREEN) - Rect.Right + Rect.Left) div 2,
	(GetSystemMetrics(SM_CYSCREEN) - Rect.Bottom + Rect.Top) div 2,
		0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
end;

initialization
{$IFDEF SCREENSAVER }
	special_config_mode_flag := false;
{$ENDIF }
end.
